/* IMPORT */
/* MAIN */
class WorkerBackend {
    /* CONSTRUCTOR */
    constructor() {
        /* VARIABLES */
        this.methods = {};
        addEventListener('message', this.message.bind(this));
    }
    /* API */
    exec(method, args) {
        const fn = this.methods[method];
        const result = new Promise(resolve => resolve(fn.apply(undefined, args)));
        const onSuccess = (value) => {
            try {
                postMessage({ type: 'result', value });
            }
            catch (error) {
                onError(error);
            }
        };
        const onError = (error) => {
            error = (error instanceof Error) ? error : (typeof error === 'string' ? new Error(error) : new Error());
            const { message, name, stack } = error;
            postMessage({ type: 'result', error: { message, name, stack } });
        };
        result.then(onSuccess, onError);
    }
    init(methods) {
        this.register(methods);
        postMessage({ type: 'ready' });
    }
    message(message) {
        if (message.data.type === 'init')
            return this.init(message.data.methods);
        if (message.data.type === 'exec')
            return this.exec(message.data.method, message.data.args);
    }
    register(methods) {
        if (typeof methods === 'string') { // Serialized function that returns the methods
            const fns = new Function(methods)();
            for (const method in fns) {
                this.methods[method] = fns[method];
            }
        }
        else { // Serialized methods map
            for (const method in methods) {
                const fn = new Function(`return (${methods[method]})`)();
                this.methods[method] = fn;
            }
        }
    }
}
/* INIT */
new WorkerBackend();
export {};
